@@ -15,9 +15,10 @@ from damask_parse import (
 )
 from damask import DADF5
 from damask_parse.utils import (
-    get_header,
+    get_header_lines,
     parse_damask_spectral_version_info,
     volume_element_from_2D_microstructure,
+    add_volume_element_missing_texture,
 )
 from damask_parse import __version__ as damask_parse_version
 
@@ -33,62 +34,35 @@ from matflow_damask.utils import get_HDF5_incremental_quantity
 
 
 @input_mapper(
-    input_file='orientations_coordinate_system.txt',
-    task='generate_microstructure_seeds',
-    method='random'
-)
-def write_orientations_coordinate_system(path, orientations_coordinate_system):
-    with Path(path).open('w') as handle:
-        json.dump(orientations_coordinate_system, handle)
-
-
-@input_mapper(
-    input_file='orientations_coordinate_system.txt',
+    input_file='orientation_coordinate_system.json',
     task='generate_volume_element',
     method='random_voronoi'
 )
-@input_mapper(
-    input_file='orientations_coordinate_system.txt',
-    task='generate_volume_element',
-    method='random_voronoi_from_orientations'
-)
-def write_orientations_coordinate_system_from_seeds(path, microstructure_seeds):
-    write_orientations_coordinate_system(
-        path,
-        microstructure_seeds['orientations_coordinate_system']
-    )
-
-
-@input_mapper(
-    input_file='model_coordinate_system.txt',
-    task='generate_volume_element',
-    method='random_voronoi'
-)
-@input_mapper(
-    input_file='model_coordinate_system.txt',
-    task='generate_volume_element',
-    method='random_voronoi_from_orientations'
-)
-def write_model_coordinate_system(path, model_coordinate_system):
+def write_orientation_coordinate_system_from_seeds(path, microstructure_seeds):
     with Path(path).open('w') as handle:
-        json.dump(model_coordinate_system, handle)
+        json.dump(microstructure_seeds['orientation_coordinate_system'], handle)
 
 
-def read_orientations_coordinate_system(path):
-    with Path(path).open('r') as handle:
-        return json.load(handle)
+@input_mapper(
+    input_file='orientation_coordinate_system.json',
+    task='generate_volume_element',
+    method='random_voronoi_from_orientations'
+)
+def write_orientation_coordinate_system_from_orientations(path, orientations):
+    with Path(path).open('w') as handle:
+        json.dump(orientations['orientation_coordinate_system'], handle)
 
 
-def read_model_coordinate_system(path):
+def read_orientation_coordinate_system(path):
     with Path(path).open('r') as handle:
         return json.load(handle)
 
 
 @output_mapper('microstructure_seeds', 'generate_microstructure_seeds', 'random')
-def read_seeds_from_random(seeds_path, orientations_coord_sys_path=None):
+def read_seeds_from_random(seeds_path, orientation_coordinate_system, phase_label):
     'Parse the file from the `seeds_fromRandom` DAMASK command.'
 
-    header_lns = get_header(seeds_path)
+    header_lns = get_header_lines(seeds_path)
     num_header_lns = len(header_lns)
 
     grid_size = None
@@ -108,24 +82,33 @@ def read_seeds_from_random(seeds_path, orientations_coord_sys_path=None):
         'eulers': eulers,
         'grid_size': grid_size,
         'random_seed': random_seed,
+        'orientation_coordinate_system': orientation_coordinate_system,
+        'phase_label': phase_label,
     }
-    if orientations_coord_sys_path:
-        ori_coord_sys = read_orientations_coordinate_system(orientations_coord_sys_path)
-        out.update({'orientations_coordinate_system': ori_coord_sys})
 
     return out
 
 
 @output_mapper('volume_element', 'generate_volume_element', 'random_voronoi')
 @output_mapper('volume_element', 'generate_volume_element', 'random_voronoi_from_orientations')
-def read_damask_geom(geom_path, orientations_coord_sys_path=None, model_coord_sys_path=None):
+def read_damask_geom(geom_path, ori_coord_system_path, phase_label_path,
+                     model_coordinate_system, buffer_phase_label):
+
     volume_element = read_geom(geom_path)
-    if orientations_coord_sys_path:
-        ori_coord_sys = read_orientations_coordinate_system(orientations_coord_sys_path)
-        volume_element.update({'orientations_coordinate_system': ori_coord_sys})
-    if model_coord_sys_path:
-        mod_coord_sys = read_model_coordinate_system(model_coord_sys_path)
-        volume_element.update({'model_coordinate_system': mod_coord_sys})
+    volume_element['model_coordinate_system'] = model_coordinate_system
+
+    ori_coord_sys = read_orientation_coordinate_system(ori_coord_system_path)
+    volume_element['orientation_coordinate_system'] = ori_coord_sys
+
+    with Path(phase_label_path).open('r') as handle:
+        phase_label = handle.read().strip()
+
+    phase_labels = [phase_label]
+    if buffer_phase_label:
+        phase_labels.append(buffer_phase_label)
+        add_volume_element_missing_texture(volume_element)
+    volume_element['phase_labels'] = np.array(phase_labels)
+
     return volume_element
 
 
@@ -141,8 +124,25 @@ def write_damask_geom(path, volume_element):
 
 
 @input_mapper('material.config', 'simulate_volume_element_loading', 'CP_FFT')
-def write_damask_material(path, material_properties, volume_element):
-    write_material_config(material_properties, Path(path).parent, volume_element)
+def write_damask_material(path, homogenization_schemes, homogenization_labels,
+                          volume_element, single_crystal_parameters, phases,
+                          texture_alignment_method):
+
+    # Merge single-crystal properties into phases:
+    for phase_label in phases.keys():
+        SC_params_name = phases[phase_label].pop('single_crystal_parameters', None)
+        if SC_params_name:
+            phases[phase_label].update(**single_crystal_parameters[SC_params_name])
+
+    write_material_config(
+        homog_schemes=homogenization_schemes,
+        phases=phases,
+        dir_path=Path(path).parent,
+        volume_element=volume_element,
+        separate_parts=True,
+        homog_labels=homogenization_labels,
+        texture_alignment_method=texture_alignment_method,
+    )
 
 
 @output_mapper('volume_element_response', 'simulate_volume_element_loading', 'CP_FFT')
@@ -165,7 +165,7 @@ def read_damask_hdf5_file(hdf5_path, incremental_data, operations=None):
                     sum_along_axes : int, optional
                         If specified, take the sum the array along this axis.
                     mean_along_axes: int, optional
-                        If specified, take the mean average of the array along this axis.        
+                        If specified, take the mean average of the array along this axis.
     operations : list of dict, optional
         List of methods to invoke on the DADF5 object. This is a list of dicts with the
         following keys:
@@ -225,6 +225,13 @@ def read_damask_hdf5_file(hdf5_path, incremental_data, operations=None):
     return volume_element_response
 
 
+@input_mapper('phase_label.txt', 'generate_volume_element', 'random_voronoi')
+@input_mapper('phase_label.txt', 'generate_volume_element', 'random_voronoi_from_orientations')
+def write_phase_label(path, microstructure_seeds):
+    with Path(path).open('w') as handle:
+        handle.write(f'{microstructure_seeds["phase_label"]}\n')
+
+
 @input_mapper('orientation.seeds', 'generate_volume_element', 'random_voronoi')
 def write_microstructure_seeds(path, microstructure_seeds):
 
@@ -281,7 +288,7 @@ def volume_element_from_microstructure_image(microstructure_image, depth, image_
     out = {
         'volume_element': volume_element_from_2D_microstructure(
             microstructure_image,
-            depth, 
+            depth,
             image_axes
         )
     }
